# lognews: A utility to grab Usenet article headers from a NNTP server
# and log them to a database.
#
# Copyright (c) Rayner Lucas 2004, 2010.
#
# This file is part of lognews.
#
# lognews is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lognews is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with lognews.  If not, see <http://www.gnu.org/licenses/>.


# NewsDaemon: Handle interactions with the OS (loading config data,
#             saving config data, signals) and the main timing loop.

import cPickle, newscollector, signal, string, sys, time


class NewsDaemon:
	# Set up variables to sensible default values, in case we can't
	# get a value from the config file.
	newshost = 'localhost'
	newsport = 119  # default NNTP port
	newsuser = 'nobody'
	newspass = ''
	dbhost = 'localhost'
	dbport = 3306	# default MySQL port
	dbuser = 'nobody'
	dbpass = ''
	dbname = 'news'
	pause = 10
	lastchecktime = time.time()

	def __init__(self):
		signal.signal(signal.SIGINT, self.signal_handler)
		signal.signal(signal.SIGTERM, self.signal_handler)

		# Find out when we last checked for new news.
		f = open('checktime', 'r')
		self.lastchecktime = cPickle.load(f)

		# Read configuration file.
		self.readConfig()


	def readConfig(self):
		f = open('lognews.conf', 'r')
		line = f.readline()
		while line:
			if line[0] not in ('#', '\n'):
				splitline = string.split(line)
				if splitline[0] == 'newshost':
					self.newshost = string.strip(splitline[1])
				elif splitline[0] == 'newsport':
					self.newsport = int(string.strip(splitline[1]))
				elif splitline[0] == 'newsuser':
					self.newsuser = string.join(splitline[1:])
				elif splitline[0] == 'newspass':
					self.newspass = string.join(splitline[1:])
				if splitline[0] == 'dbhost':
					self.dbhost = string.strip(splitline[1])
				elif splitline[0] == 'dbport':
					self.dbport = int(string.strip(splitline[1]))
				elif splitline[0] == 'dbuser':
					self.dbuser = string.join(splitline[1:])
				elif splitline[0] == 'dbpass':
					self.dbpass = string.join(splitline[1:])
				elif splitline[0] == 'dbname':
					self.dbname = string.join(splitline[1:])
				elif splitline[0] == 'pause':
					self.pause = int(string.strip(splitline[1]))
			line = f.readline()


	def start(self):
		nc = newscollector.NewsCollector(self, self.newshost, self.newsport, \
				self.newsuser, self.newspass, self.dbhost, self.dbport, \
				self.dbuser, self.dbpass, self.dbname)

		# Main program loop
		while 1:
			checktime = time.time()
			success = nc.DoChecks(self.lastchecktime)
			if (success):
				print 'NewsDaemon: checks successful, updating check time...'
				self.lastchecktime = checktime

			print 'NewsDaemon: sleeping...'
			time.sleep(self.pause)
			print 'NewsDaemon: waking...'


	# cleanup: Do everything that needs doing before exiting.
	def cleanup(self):
		# write lastchecktime to file
		print 'Writing checktime to file.'
		f = open('checktime', 'w')
		cPickle.dump(self.lastchecktime, f)


	# signal_handler: trap external signals sent to the program.
	#
	# Signals currently handled are:
	# SIGTERM: call cleanup(), then exit successfully.
	# SIGINT: call cleanup(), then exit successfully.
	def signal_handler(self, signal, frame):
		print 'SIGINT or SIGTERM caught in lognews newsdaemon, exiting...'
		self.cleanup()
		sys.exit(0)
